package coffeetronix;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.AbstractButton;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.UIManager;

import coffeetronix.diagram.DiagramPanel;
import coffeetronix.diagram.Shape;

/**
 * The coffeetronix main class.
 * I hate my ErrorSystem class. Please remove it. 
 * 
 * @author Rene Stoeckel (rene.stoeckel-at-web.de)
 */

public class Coffeetronix extends JFrame
{
	/** Panel that paints the class diagram */
	public DiagramPanel diagrampanel= new DiagramPanel();

	/** 
	 * Default name of the Coffeetronix config file.
	 */
	private String CONFIG_NAME= "coffeetronix.cfg";

	public ProjectsGUI projectslist= new ProjectsGUI(new JavaProject(diagrampanel));

	private Config config= new Config();
	
	public LogWindow logwindow= new LogWindow("Coffeetronix messages");

	private JMenuBar menuBar;
	private JMenu menu;

	private String projects[]= new String[10];
	private int numrecent= 0;

	/** 
	 * Initializes coffeetronix and loads its ugly icons.
	 */
	private void init()
	{
		Toolkit.getDefaultToolkit().beep();
		this.setFont(new java.awt.Font(Config.fontname, Font.PLAIN, Config.fontsize));

		setTitle("Coffeetronix V0.2a, Rene Stoeckel and others");

		// load some icons
		MediaTracker mt= new MediaTracker(this);
		Shape.boxblueImage= getToolkit().getImage("icons/a.gif");
		Shape.boxpinkImage= getToolkit().getImage("icons/m.gif");
		Shape.privImage= getToolkit().getImage("icons/priv.gif");
		Shape.protImage= getToolkit().getImage("icons/prot.gif");

		Shape.boxblueImageGray= getToolkit().getImage("icons/a-gray.gif");
		Shape.boxpinkImageGray= getToolkit().getImage("icons/m-gray.gif");
		Shape.privImageGray= getToolkit().getImage("icons/priv-gray.gif");
		Shape.protImageGray= getToolkit().getImage("icons/prot-gray.gif");

		Shape.treeImage= getToolkit().getImage("icons/component.gif");

		mt.addImage(Shape.boxblueImage, 0);
		mt.addImage(Shape.boxpinkImage, 0);
		mt.addImage(Shape.privImage, 0);
		mt.addImage(Shape.protImage, 0);
		mt.addImage(Shape.boxblueImageGray, 0);
		mt.addImage(Shape.boxpinkImageGray, 0);
		mt.addImage(Shape.privImageGray, 0);
		mt.addImage(Shape.protImageGray, 0);
		mt.addImage(Shape.treeImage, 0);

		try
		{
			//wait until image is loaded.
			mt.waitForAll();
		}
		catch (InterruptedException e)
		{
			ErrorSystem.fatalerror("could not load icons in ./icons");
		}
		
		ErrorSystem.info("Hello World!");

		addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent event)
			{
				System.exit(0);
			}
		});

		diagrampanel.init();

		JPanel p= new JPanel(new BorderLayout());
		p.add("North", new CoffeeToolBar(this));
		p.add("Center", diagrampanel);

		getContentPane().add("Center", p);

		// load config
		//config.load("coffeetronix.cfg");

		show();
	}

	/** this is needed to enable keyevents */
	public boolean isFocusTraversable()
	{
		return true;
	}


	/** this is the main function of coffeetronix */
	public static void main(String args[])
	{

		try
		{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}
		catch (Exception e)
		{
		};

		Coffeetronix app= new Coffeetronix();
		app.setSize(800, 600);
		app.setVisible(true);
		app.init();
	}
}
