package coffeetronix;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.AbstractButton;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import coffeetronix.diagram.DiagramPanel;
import coffeetronix.diagram.Shape;
import coffeetronix.diagram.ShapeInfo;

/**
 * 
 * A Window where you can edit the comment shapes of the diagram.
 * Comment shapes are not yet linked with the source code, instead 
 * these comments are saved in the project xml file.
 * 
 * @author Rene Stoeckel (rene.stoeckel-at-web.de)
 *  
 */

public class CommentEditor extends JFrame implements ActionListener
{
	private JTextArea ta= new JTextArea("");

	/** where to send data to when ok is pressed */
	private DiagramPanel panel;

	/** the shape it belongs to */
	private Shape shape= null;

	/** Signs, if the shape has to be created in the panel. */
	private boolean isnew= true;

	public CommentEditor(String title, DiagramPanel apanel)
	{
		super(title);
		panel= apanel;
		addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent event)
			{
				dispose();
			}
			public void windowDeactivated(WindowEvent e)
			{
				dispose();
			}
		});

		Container container;

		ta.setTabSize(4);
		ta.setFont(new Font("Times", Font.PLAIN, 12));
		setSize(400, 150);
		container= this.getContentPane();
		container.setLayout(new BorderLayout());

		JScrollPane sp= new JScrollPane(ta);
		container.add(sp, BorderLayout.CENTER);

		JPanel p= new JPanel();
		JButton b;

		b= new JButton("Ok");
		b.setActionCommand("ok");
		b.addActionListener(this);
		p.add(b);

		b= new JButton("Cancel");
		b.setActionCommand("cancel");
		b.addActionListener(this);
		p.add(b);

		container.add(p, BorderLayout.SOUTH);

		ta.setText("Example-\ntext!");
	}

	public void setShape(Shape ashape, boolean isnewflag)
	{
		isnew= isnewflag;
		shape= ashape;
	}

	public void actionPerformed(ActionEvent e)
	{
		String command= ((AbstractButton) e.getSource()).getActionCommand();

		if (command.equals("ok"))
		{
			int index= 0;

			try
			{
				panel.updateCommentShape(
					ShapeInfo.createFromString(ta.getText(), ta.getLineCount()),
					shape,
					isnew);
				hide();
			}
			catch (Exception e1)
			{
				ErrorSystem.error("CommentEditor.commandok(): " + e1);
			}
		}
		else
			dispose();
	}

	public JTextArea getTextArea()
	{
		return ta;
	}
}
