package coffeetronix;

import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;

import coffeetronix.diagram.*;

/**
 * Holds information of a java class (fields, methods, access modifiers, 
 * nested classes, and much more).
 * 
 * @author Rene Stoeckel (rene.stoeckel-at-web.de)
 */

public class JavaClass
{
	private Shape shape = null;

	protected String extendsstr = "";
	protected String name = "";

	private LinkedList nestedclasses = new LinkedList();
	private LinkedList imports = new LinkedList();
	private LinkedList attributes = new LinkedList();
	private LinkedList methods = new LinkedList();

	private JavaUnit javaunit;

	public int beginline;
	public int endline;
	public int protection;
	public boolean isfinal;
	public boolean isstatic;
	public boolean isabstract;
	public boolean ispublic;
	public boolean isInterface = false;


	class JavaAttributeComparator implements Comparator
	{
		public int compare(Object o1, Object o2)
		{
			return ((JavaAttribute) o1).compareTo(((JavaAttribute) o2));
		}
	}

	class JavaMethodComparator implements Comparator
	{
		public int compare(Object o1, Object o2)
		{
			return ((JavaMethod) o1).compareTo(((JavaMethod) o2));
		}
	}

	public void setShape(Shape shape)
	{
		this.shape = shape;
	}

	public Shape getShape()
	{
		return shape;
	}

	public JavaClass(JavaUnit javaunit)
	{
		this.javaunit = javaunit;
	}

	public void useAsInterface(boolean yesno)
	{
		isInterface = yesno;
	}

	public boolean usedAsInterface()
	{
		return isInterface;
	}

	public void addNestedClass(JavaClass javaclass)
	{
		nestedclasses.add(javaclass);
	}

	
	public void setExtends(String extendsstr)
	{
		this.extendsstr = extendsstr;
	}

	public Iterator getNestedClasses()
	{
		return nestedclasses.iterator();
	}



	public void setName(String name)
	{
		this.name = name;
	}

	public String getName()
	{
		return name;
	}



	public void setbeginline(int beginline)
	{
		this.beginline = beginline;
	}

	public void setendline(int endline)
	{
		this.endline = endline;
	}



	public void addMethod(JavaMethod method)
	{
		methods.add(method);
	}


	public void addAttribute(JavaAttribute attr)
	{
		// multiple attributes ?
		attributes.add(attr);
	}



	public int getProtection()
	{
		return protection;
	}


	public JavaUnit getUnit()
	{
		return javaunit;
	}


	public String toString()
	{
		return "javaclass: " + name;
	}

	/** Fix classnames recursively (include package names). */
	public void fixnestednames(String father)
	{
		if (father.length() > 0)
			name = father + "." + name;


		for (Iterator iter = nestedclasses.iterator(); iter.hasNext(); )
		{
			JavaClass jclass = (JavaClass) iter.next();
			jclass.fixnestednames(name);
		}
	}


	/** Returns all data of a class as String[]. */
	public ShapeInfo[] getData(boolean showprotected)
	{
		Collections.sort(attributes, new JavaAttributeComparator());
		Collections.sort(methods, new JavaMethodComparator());

		LinkedList arrlist = new LinkedList();

		if (extendsstr.length() != 0 )
			arrlist.add(new ShapeInfo(javaunit.getPackage() + name + ": " + extendsstr, 4, 3));
		else
			arrlist.add(new ShapeInfo(javaunit.getPackage() + name, 4, 3));
		arrlist.add(new ShapeInfo("", 0, 10));


		// public attributes
		for (Iterator iter = attributes.listIterator(); iter.hasNext(); )
		{
			JavaAttribute attr = (JavaAttribute)iter.next();

			// single attributes
			for (Iterator iter2 = attr.getSingleAttributes().iterator();
					iter2.hasNext();)
			{
				JavaAttribute attr2 = (JavaAttribute)iter2.next();
				if (attr2.getProtection() == 0)
					arrlist.add(new ShapeInfo("" + attr2.toString() + ";",
						attr2.getProtection(), 2));
			}
		}

		// protected and private attributes
		for (Iterator iter = attributes.listIterator(); iter.hasNext(); )
		{
			JavaAttribute attr = (JavaAttribute)iter.next();

			// single attributes
			for (Iterator iter2 = attr.getSingleAttributes().iterator();
					iter2.hasNext();)
			{
				JavaAttribute attr2 = (JavaAttribute)iter2.next();
				if ((attr2.getProtection() != 0) && showprotected)
					arrlist.add(new ShapeInfo("" + attr2.toString() + ";",
						attr2.getProtection(), 2));
			}
		}


		// separator
		arrlist.add(new ShapeInfo("", 0, 10));

		// public methods
		for (Iterator iter = methods.listIterator(); iter.hasNext(); )
		{
			JavaMethod javamethod = (JavaMethod)iter.next();
			if (javamethod.getProtection() == 0)
				arrlist.add(new ShapeInfo("" + javamethod.toString() + ";",
					javamethod.getProtection(), 1));
		}

		// protected and private methods
		for (Iterator iter = methods.listIterator(); iter.hasNext(); )
		{
			JavaMethod javamethod = (JavaMethod)iter.next();
			if ((javamethod.getProtection() != 0) && showprotected)
				arrlist.add(new ShapeInfo("" + javamethod.toString() + ";",
					javamethod.getProtection(), 1));
		}



		// convert list to array
		ShapeInfo arr2[] = new ShapeInfo[arrlist.size()];

		int n = 0;
		for (Iterator iter = arrlist.iterator(); iter.hasNext();)
		{
			arr2[n] = (ShapeInfo) iter.next();
			n++;
		}

		return arr2;
	}
}

