package coffeetronix;

import java.util.Iterator;
import java.util.LinkedList;

/** Holds parsed information about java methods 
 * 
 * 
 * @author Rene Stoeckel (rene.stoeckel-at-web.de)
 * 
 */

public class JavaMethod
{

	/** 0=public, 1=protected, 2=private */
	public int protection= 0; 
	public boolean isstatic= false;
	public boolean isfinal= false;
	private boolean istransient= false;
	private boolean isvolatile= false;
	public boolean isnative= false;
	public boolean issynchronized= false;
	public boolean isabstract= false;

	private int beginline= 0;
	private int endline= 0;

	private String name;
	private String returntype= "";

	private LinkedList parameters= new LinkedList();

	public void addParameter(String namestr, String typestr)
	{
		parameters.add(new JavaMethodParameter(namestr, typestr));
	}

	public void setReturnType(String str)
	{
		returntype= str;
	}

	public void setName(String str)
	{
		name= str;
	}

	public void setbeginline(int line)
	{
		this.beginline= line;
	}

	public void setendline(int line)
	{
		this.endline= line;
	}

	public int getProtection()
	{
		return protection;
	}

	public void setProtection(int p)
	{
		protection= p;
	}

	public String toString()
	{
		if (parameters.size() > 3)
			return returntype + " " + name + "(..)";
		else
		{
			String params= "(";

			for (Iterator iter= parameters.iterator(); iter.hasNext();)
			{
				JavaMethodParameter param= (JavaMethodParameter) iter.next();
				params += param.type + " " + param.name;
				if (iter.hasNext())
					params += ", ";
			}
			params += ")";
			return returntype + " " + name + params;
		}
	}

	public int compareTo(JavaMethod m2)
	{
		return toString().compareTo(m2.toString());
	}
}
