package coffeetronix.diagram;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionAdapter;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedList;

import javax.imageio.ImageIO;
import javax.swing.AbstractButton;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JScrollBar;

import coffeetronix.CommentEditor;
import coffeetronix.Config;
import coffeetronix.ErrorSystem;
import coffeetronix.JavaClass;
import coffeetronix.JavaProject;

/**
 *
 * This is the panel that paints the class diagrams.
 * Well, its already a stupid overcrowded monster class.
 * The Innerdiagrammpanel is responsible for the drawing 
 * of the diagram panel.
 * 
 * @author Rene Stoeckel (rene.stoeckel-at-web.de)
 */

public class DiagramPanel extends JComponent implements ActionListener, AdjustmentListener
{

	private static boolean showprotectedclasses= true;
	/** A mouse handling mode */
	private final static int MOUSE_MOVESHAPES= 0;

	/** Another mouse handling mode */
	private final static int MOUSE_SCROLLVIEW= 1;

	/** Counts shape-IDs, (used for creating names for comment shapes). */
	private static int shapeid= 0;

	/** 
	 * When the view is not scrolled by scrollbars and the scrollbars are adjusted, 
	* their scrollevents have 2 be disabled. I hate this. 
	*/
	private boolean allowScrollEvents= true;

	/** When in applet mode, some menu buttons are disabled. */
	public boolean appletmode= false;

	/** Flag to disable drawing (during load) */
	private boolean candisplay= true;

	/**
	 * Comments you want to add to a class you
	 * enter comments in this window
	 */
	private CommentEditor commenteditor= new CommentEditor("Comment Editor Window", this);

	/** Popup menu displayed for comment shapes */
	private JPopupMenu commentpopup= new JPopupMenu();

	/** Size of the drawing area */
	private Dimension drawareasize;

	/** Flag to control the popup menu */
	private boolean enablepopup= true;

	/** Font size parameters for geometry */
	private FontMetrics fm;

	private InnerDiagramPanel innerpanel= new InnerDiagramPanel();

	/** Self made double click- measurement */
	private long lastclicktime= 0;

	/** All relations between the java classes */
	private LinkedList links= new LinkedList();

	/**
	 * Mouse selection Mode
	 */
	private int mousemode= 0;

	/** Popup menu displayed when multiple class shapes are selected */
	private JPopupMenu multipopup= new JPopupMenu();

	/** Font of diagram elements, see Config.class */
	private java.awt.Font myFont;

	/** Double buffer graphics */
	private Graphics offgraphics;

	/** Double buffering */
	private Image offscreen;

	/** Double buffer size */
	private Dimension offscreensize;

	/** X-Coordinate of View offset, all class elements are
	 * painted relative to it */
	private int offsetX= 0;

	/** X-Coordinate of View offset, all class elements are
	 * painted relative to it */
	private int offsetY= 0;

	/** The picked and shift-marked shape(s) */
	private Shape pick, shiftpick;

	/** X position of mouse event */
	private int pressedX= 0;

	/** Y position of mouse event */
	private int pressedY= 0;
	private JScrollBar sbarhorizontal= new JScrollBar(JScrollBar.HORIZONTAL);
	private JScrollBar sbarvertical= new JScrollBar(JScrollBar.VERTICAL);

	/** All java class shapes */
	private LinkedList shapes= new LinkedList();

	/** Class to manage multiple selection of class shapes */
	private Selection selection= new Selection(shapes);

	/** Popup menu displayed when one class shape is selected */
	private JPopupMenu singlepopup= new JPopupMenu();

	/** Size of visible drawing area, needed to compute
	 *  scrollbar parameters */
	private VisualRectangle vrect= new VisualRectangle();

	/** Empty Constructor. */
	public DiagramPanel()
	{
	}

	class InnerDiagramPanel extends JComponent
	{
		public void paint(Graphics g)
		{
			update(g);
		}


		public void exportdraw(Graphics offgraphics)
		{
			
			offgraphics.setColor(Config.paperColor);
			offgraphics.fillRect(0, 0, drawareasize.width, drawareasize.height);

			drawGrid(offgraphics);

			offgraphics.setColor(Config.linkColor);

			//offgraphics.drawString("hello, a short message.here comes tha newline!",20,20);

			// draw links
			for (Iterator iter= links.iterator(); iter.hasNext();)
			{
				Link link= (Link)iter.next();
				if ((link.from.shape.isDirty()) || (link.to.shape.isDirty()))
				{
					link.from.shape.registerLink(link.to);
					link.to.shape.registerLink(link.from);
				}

				paintLink(offgraphics, link.to, link.from);
			}

			// draw link arrows
			for (Iterator iter= links.iterator(); iter.hasNext();)
			{
				Link link= (Link)iter.next();
				paintLinkInfo(link.to, offgraphics);
				drawLinkEnd(offgraphics, link.from, link.type);
			}

			// draw classes
			java.awt.geom.Rectangle2D rect= new Rectangle(0, 0, 0, 0);

			for (Iterator iter= shapes.iterator(); iter.hasNext();)
			{
				Shape n= (Shape)iter.next();
				Image image= n.getImage();

				offgraphics.drawImage(
					image,
					offsetX + n.getTopLeft().x,
					offsetY + n.getTopLeft().y,
					n.getDimension().width,
					n.getDimension().height,
					null);

				if (n.isDirty())
				{
					rect= rect.createUnion(n.getRectangle());
					n.setDirty(false);
				}
			}

			offgraphics.setColor(Color.black);
							offgraphics.drawString("Program: Coffeetronix V0.3, (2003)", 10,10);
							offgraphics.drawString("License: GNU general public licence (GPL)", 10,20);
							offgraphics.drawString("Author: rene.stoeckel@web.de and others", 10,30);
							
			// draw a border			
			offgraphics.setColor(Config.gridColor);

			offgraphics.drawRect(0, 0, drawareasize.width - 1, drawareasize.height - 1);
			

		}


		/** Updates all graphics */
		public void update(Graphics g)
		{
			try
			{
				drawareasize= getSize();

				// show load screen
				if (!candisplay)
				{
					g.setColor(Config.paperColor);
					g.fillRect(0, 0, drawareasize.width, drawareasize.height);
					g.setFont(new java.awt.Font("TimesRoman", Font.PLAIN, 36));
					g.setColor(Color.black);
					g.drawString(
						new String("loading, please wait..."),
						drawareasize.width / 3,
						drawareasize.height / 3);
					g.setFont(myFont);
					return;
				}

				// double buffer, lazy instantiation 
				if ((offscreen == null)
					|| (drawareasize.width != offscreensize.width)
					|| (drawareasize.height != offscreensize.height))
				{
					setDimension(drawareasize);
				}

				// clear double buffer
				offgraphics.setColor(Config.paperColor);
				offgraphics.fillRect(0, 0, drawareasize.width, drawareasize.height);

				drawGrid(offgraphics);

				offgraphics.setColor(Config.linkColor);

				//offgraphics.drawString("hello, a short message.here comes tha newline!",20,20);

				// draw links
				for (Iterator iter= links.iterator(); iter.hasNext();)
				{
					Link link= (Link)iter.next();
					if ((link.from.shape.isDirty()) || (link.to.shape.isDirty()))
					{
						link.from.shape.registerLink(link.to);
						link.to.shape.registerLink(link.from);
					}

					paintLink(offgraphics, link.to, link.from);
				}

				// draw link arrows
				for (Iterator iter= links.iterator(); iter.hasNext();)
				{
					Link link= (Link)iter.next();
					paintLinkInfo(link.to, offgraphics);
					drawLinkEnd(offgraphics, link.from, link.type);
				}

				// draw classes
				java.awt.geom.Rectangle2D rect= new Rectangle(0, 0, 0, 0);

				for (Iterator iter= shapes.iterator(); iter.hasNext();)
				{
					Shape n= (Shape)iter.next();
					Image image= n.getImage();

					offgraphics.drawImage(
						image,
						offsetX + n.getTopLeft().x,
						offsetY + n.getTopLeft().y,
						n.getDimension().width,
						n.getDimension().height,
						null);

					if (n.isDirty())
					{
						rect= rect.createUnion(n.getRectangle());
						n.setDirty(false);
					}
				}

				offgraphics.setColor(Color.black);
				offgraphics.drawString("Program: Coffeetronix V0.3, (2003)", 10,10);
				offgraphics.drawString("License: GNU general public licence (GPL)", 10,20);
				offgraphics.drawString("Author: rene.stoeckel@web.de and others", 10,30);

				// draw a border				
				offgraphics.setColor(Config.gridColor);
				offgraphics.drawRect(0, 0, drawareasize.width - 1, drawareasize.height - 1);

				// flush offscreen
				g.drawImage(offscreen, 0, 0, null);

				offgraphics.setColor(Config.gridColor);

				if (shapes.size() > 0)
					g.drawRect(
						offsetX + vrect.getLocation().x,
						offsetY + vrect.getLocation().y,
						vrect.getSize().width,
						vrect.getSize().height);
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
	}

	/** Handling of key events is prepared, but not used much */
	private class MyKeyAdapter extends KeyAdapter
	{
		/** supports CTRL-A to select all shapes */
		public void keyTyped(KeyEvent e)
		{
			//ErrorSystem.info("key pressed: " + e.getKeyCode());
			if (e.getKeyChar() == 0x01)
			{
				selection.selectAll();
				repaint();
			}
		}
	}

	/** Handling the Mouse Events for navigation, shape movement etc.*/

	private class MyMouseAdapter extends MouseMotionAdapter implements MouseListener
	{

		/** Empty function.*/
		public void mouseClicked(MouseEvent e)
		{
		}

		/** Handling mouse movements */
		public void mouseDragged(MouseEvent e)
		{

			if (mousemode == MOUSE_MOVESHAPES)
			{
				if (pick != null)
				{
					selection.translate(1 * (e.getX() - pressedX), 1 * (e.getY() - pressedY));

					vrect.setDirty();
				}
			}
			else
				if (mousemode == MOUSE_SCROLLVIEW)
				{
					offsetX += 4 * (e.getX() - pressedX);
					offsetY += 4 * (e.getY() - pressedY);
					if ((Math.abs(e.getX() - pressedX) + Math.abs(e.getY() - pressedY)) > 10)
						enablepopup= false;

					// update scrollers
					Dimension d1= vrect.getSize();
					int x1= d1.width - drawareasize.width;
					int y1= d1.height - drawareasize.height;
					Point pos= vrect.getLocation();

					int x= 0, y= 0;
					if (x1 != 0)
						x= - (offsetX + pos.x) * 100 / x1;
					if (y1 != 0)
						y= - (offsetY + pos.y) * 100 / y1;

					setScrollerValues(x, y);

				}
			pressedX= e.getX();
			pressedY= e.getY();
			repaint();
			e.consume();
		}
		/** Empty function.*/
		public void mouseEntered(MouseEvent e)
		{
		}

		/** Empty function.*/
		public void mouseExited(MouseEvent e)
		{
		}

		/**
		 * Switches the mouse motion listener on,
		 * looks for selection of classes and does popup the menus.
		 */
		public void mousePressed(MouseEvent e)
		{

			addMouseMotionListener(this);

			// always reset selection, when shift is not pressed.
			if (!e.isShiftDown())
			{
				selection.unselectAll();
			}

			if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) != 0)
			{

				setMouseMode(MOUSE_MOVESHAPES);

				setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));

				pick= getShape(new Point(e.getX() - offsetX, e.getY() - offsetY));

				if (pick != null)
				{
					selection.select(pick);
				}

			}
			else
			{
				// right mouse
				setMouseMode(MOUSE_SCROLLVIEW);
				setCursor(new Cursor(Cursor.HAND_CURSOR));
			}

			pressedX= e.getX();
			pressedY= e.getY();
			repaint();

			// help me: is this necessary ?
			e.consume();
			enableScrollEvents(false);
			requestFocus();
		}

		/**
		 * Switch off Mouse motion listener, handle popup menus
		 *
		 */
		public void mouseReleased(MouseEvent e)
		{
			// unsubscribe mousemove events
			removeMouseMotionListener(this);

			if (mousemode == MOUSE_MOVESHAPES)
			{

			}

			else
				if (mousemode == MOUSE_SCROLLVIEW)
				{
					// did we scroll ?
					if (enablepopup)
					{
						//no, do popup handling.

						Shape tmpshape= getShape(new Point(e.getX() - offsetX, e.getY() - offsetY));

						if (tmpshape != null)
						{
							selection.select(tmpshape);

							// multiple classes selected ?
							if (selection.size() > 1)
							{
								showPopup(e, multipopup);
							}
							else
							{
								// first shape a class shape ?
								Shape firstshape= (Shape)selection.get(0);

								if (firstshape.getType() == Shape.TYPE_CLASS)
								{
									showPopup(e, singlepopup);
								}
								else
									showPopup(e, commentpopup);
							}
						}
					}
				}

			setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			repaint();
			// do we really need this ?
			e.consume();
			enablepopup= true;
			enableScrollEvents(true);
		}

		/** Switches Popup Menu on/off */
		private void showPopup(MouseEvent e, JPopupMenu popup)
		{
			if (popup != null)
				popup.show(e.getComponent(), e.getX(), e.getY());
		}

	}

	/**
	 * Class for Managing a set of selected shapes.
	 * Needs the container class (list) of all shapes.
	 */
	private class Selection extends LinkedList
	{
		private LinkedList src;

		Selection(LinkedList src)
		{
			this.src= src;
		}

		/** Adds a Shape to the selection. Avoids double occurences. */
		private void select(Shape n)
		{
			n.selectOn();
			if (!contains(n))
				add(n);
		}

		/** Selects all shapes. Avoids double occurences.*/
		private void selectAll()
		{
			clear();
			for (Iterator iter= src.iterator(); iter.hasNext();)
			{
				Shape n= (Shape)iter.next();
				n.selectOn();
				add(n);
			}
		}

		/** Moves all shapes by a given distance. */
		private void translate(int x, int y)
		{
			for (Iterator iter= iterator(); iter.hasNext();)
			{
				Shape n= (Shape)iter.next();
				n.move2D(x, y);
			}
		}

		/** Removes a Shape from the selection. */
		private void unselect(Shape n)
		{
			n.selectOff();
			remove(n);
		}

		/** Clears the selection */
		private void unselectAll()
		{
			selection.clear();
			for (Iterator iter= src.iterator(); iter.hasNext();)
			{
				Shape n= (Shape)iter.next();
				n.selectOff();
			}
		}
	}

	/**
	 *  Calculates the total size of the sheet for scroll handling.
	 */

	private class VisualRectangle
	{

		private boolean dirty= true;
		private LinkedList nodes;
		private Rectangle rect= new Rectangle(0, 0, 0, 0);

		private void fix()
		{
			dirty= false;

			rect= new Rectangle(0, 0, 0, 0);

			for (Iterator iter= shapes.iterator(); iter.hasNext();)
			{
				Shape n= (Shape)iter.next();
				rect= (rect.union(n.getRectangle()));
			}

			// add 50 pixels additional space, no real reasen,
			// gives a kind of border.

			rect.width += 100;
			rect.height += 100;
			rect.x -= 50;
			rect.y -= 50;
		}

		public Point getLocation()
		{
			if (dirty)
				fix();
			return rect.getLocation();
		}

		public Dimension getSize()
		{
			if (dirty)
				fix();
			return rect.getSize();
		}

		private void setDirty()
		{
			dirty= true;
		}

	}

	public static JButton createButton(String text, String command, ActionListener listener)
	{
		JButton m= new JButton(text);
		m.setActionCommand(command);
		m.addActionListener(listener);
		return m;
	}

	public static JMenuItem createMenuItem(String text, String command, ActionListener listener)
	{
		JMenuItem m= new JMenuItem(text);
		m.setActionCommand(command);
		m.addActionListener(listener);
		return m;
	}

	/** Processes all popup menu commands. */
	public void actionPerformed(ActionEvent e)
	{
		String command= ((AbstractButton)e.getSource()).getActionCommand();

		// full display of a class
		if (command.equals("expand"))
		{
			Shape shape= (Shape)selection.getFirst();
			if (shape != null)
			{
				shape.toggleExpand();
				repaint();
			}

			return;
		}
		else

			// adds an association
			if (command.equals("addass"))
			{
				addLink(0);
				return;
			}

		// adds a generalization
		else
			if (command.equals("addgen"))
			{
				addLink(1);
				return;
			}

		// removes a link
		else
			if (command.equals("removelink"))
			{
				removeLink();

				return;
			}

			else
				if (command.equals("addcomment"))
				{
					Shape shape= (Shape)selection.getFirst();
					if (shape != null)
					{
						commenteditor.getTextArea().setText("");

						// is a new to be inserted, so true
						commenteditor.setShape(shape, true);
						commenteditor.show();
					}
					return;
				}
		// opens the comment editor
		else
			if (command.equals("commentedit"))
			{
				commenteditor.setShape((Shape)selection.getFirst(), false);
				commenteditor.getTextArea().setText("");
				ShapeInfo ti[]= ((Shape)selection.getFirst()).getData();
				for (int n= 0; n < ti.length; n++)
				{
					commenteditor.getTextArea().append(ti[n].getText() + "\n");
				}
				commenteditor.show();
			}

		// removes a comment shape
		else
			if (command.equals("commentremove"))
			{
				Shape n0= (Shape)selection.getFirst();
				Shape n1= (Shape)getShape(n0);
				if (n1 != null)
					removeLink(n0, n1);
				shapes.remove(n0);
			}

			else
				if (command.equals("colorgray"))
				{
					for (Iterator iter= selection.iterator(); iter.hasNext();)
					{
						Shape shape= (Shape)iter.next();
						shape.setShapeColor(0);
						shape.redraw();
					}
					repaint();
				}
				else
					if (command.equals("colorblue"))
					{
						for (Iterator iter= selection.iterator(); iter.hasNext();)
						{
							Shape shape= (Shape)iter.next();
							shape.setShapeColor(1);
							shape.redraw();
						}
						repaint();
					}
					else
						if (command.equals("colorred"))
						{
							for (Iterator iter= selection.iterator(); iter.hasNext();)
							{
								Shape shape= (Shape)iter.next();
								shape.setShapeColor(2);
								shape.redraw();
							}
							repaint();
						}
						else
							if (command.equals("colorgreen"))
							{
								for (Iterator iter= selection.iterator(); iter.hasNext();)
								{
									Shape shape= (Shape)iter.next();
									shape.setShapeColor(3);
									shape.redraw();
								}
								repaint();
							}
							else
								if (command.equals("coloryellow"))
								{
									for (Iterator iter= selection.iterator(); iter.hasNext();)
									{
										Shape shape= (Shape)iter.next();
										shape.setShapeColor(4);
										shape.redraw();
									}
									repaint();
								}

								else
									ErrorSystem.error("Unknown command:" + command);
	}

	/** Adds a java class to the diagram, creates also its shape */
	public void addClass(JavaClass javaclass, JavaClass father)
	{
		Shape shape1= new Shape();
		shape1.init(this, javaclass.getName(), 0);
		shape1.setShapeInfo(javaclass.getData(showprotectedclasses));
		javaclass.setShape(shape1);

		shapes.add(shape1);

		if (father != null)
		{
			Shape n2= father.getShape();
			if (n2 != null)
				addLink(shape1, n2, 2); // nesting
		}

		vrect.setDirty();
		repaint();
	}

	/** Adds subclasses of a father class ( adds generalization links also) */
	public void addClasses(Iterator iterator, JavaClass father)
	{
		for (Iterator iter= iterator; iter.hasNext();)
		{
			JavaClass javaclass= (JavaClass)iter.next();
			addClass(javaclass, father);

			// javaclass is father of its nested classes
			addClasses(javaclass.getNestedClasses(), javaclass);
		}
	}

	/** Adds a link between 2 selected java classes (0= assiciation,
	 *  1= generalization, 2= aggregation).
	 */

	public void addLink(int type)
	{

		if (selection.size() != 2)
		{
			ErrorSystem.error("Please select 2 Nodes.");
			return;
		}
		addLink((Shape)selection.get(0), (Shape)selection.get(1), type);
		repaint();
	}

	/** Adds a link between 2 java classes (addressed by their id) */
	public void addLink(int from, int to, int type)
	{
		if (from == to)
			return;

		Shape n1= null;
		Shape n2= null;

		n1= (Shape)shapes.get(from);
		n2= (Shape)shapes.get(to);

		addLink(n1, n2, type);
	}

	/** Adds a link between 2 java classes (0= assiciation,
	 *  1= generalization, 2= aggregation)*/
	public void addLink(Shape n1, Shape n2, int type)
	{

		if ((n1 == null) || (n2 == null))
		{
			ErrorSystem.error("addLink(): node(s) not found ?!");
			return;
		}

		if (n1 == n2)
		{
			ErrorSystem.error("can not add a link to same shape!");
			return;
		}

		Link newlink= new Link(n1, n2, type);
		links.add(newlink);

		repaint();
	}

	/** Process the scroll events. */
	public void adjustmentValueChanged(AdjustmentEvent e)
	{
		if (allowScrollEvents)
			setViewArea(sbarhorizontal.getValue(), sbarvertical.getValue());
	}

	/** Cleanup function. Forgets anything */
	public void clearAll()
	{
		shapes.clear();
		selection.clear();
		links.clear();
		repaint();
		pick= null;
	}

	/** Create the popup menus. */
	private void createpopups()
	{
		//create the popup menu.
		singlepopup.add(createMenuItem("expand/collapse", "expand", this));

		singlepopup.add(createMenuItem("color gray", "colorgray", this));
		singlepopup.add(createMenuItem("color blue", "colorblue", this));
		singlepopup.add(createMenuItem("color red", "colorred", this));
		singlepopup.add(createMenuItem("color green", "colorgreen", this));
		singlepopup.add(createMenuItem("color yellow", "coloryellow", this));

		singlepopup.add(createMenuItem("add comment", "addcomment", this));
		singlepopup.add(createMenuItem("generate code", "generate", this));
		singlepopup.add(createMenuItem("info", "info", this));
		singlepopup.add(createMenuItem("edit", "singleedit", this));

		multipopup.add(createMenuItem("multi", "multi", this));
		multipopup.add(createMenuItem("add association", "addass", this));
		multipopup.add(createMenuItem("add generalization", "addgen", this));
		multipopup.add(createMenuItem("remove link", "removelink", this));
		multipopup.add(createMenuItem("edit", "multiedit", this));

		//commentpopup.add(createMenuItem("comment", "comment", this));
		commentpopup.add(createMenuItem("edit", "commentedit", this));
		commentpopup.add(createMenuItem("remove", "commentremove", this));

	}

	/** During loading it is switched off*/
	public void displayOff()
	{
		candisplay= false;
		repaint();
		//update(getGraphics());
	}

	/** After loading it is switched on again. */
	public void displayOn()
	{
		offsetX= offsetY= 0;
		candisplay= true;
		repaint();
		//update(getGraphics());

	}

	/** This draws the gray grid for better adjustment */
	private void drawGrid(Graphics g)
	{
		g.setColor(Config.gridColor);
		for (int x1= offsetX % Config.gridSize; x1 < drawareasize.width; x1 += Config.gridSize)
		{
			g.drawLine(x1, 0, x1, drawareasize.height);
		}

		for (int y1= offsetY % Config.gridSize; y1 < drawareasize.height; y1 += Config.gridSize)
		{
			g.drawLine(0, y1, drawareasize.width, y1);
		}

	}

	/** Several arrow ends are possible for drawing */
	private void drawLinkEnd(Graphics offgraphics, LinkInfo info, int type)
	{

		// polygon data
		int xarr[];
		int yarr[];

		int x= info.connector.x;
		int y= info.connector.y;
		int arrow_len= Config.arrowlength;
		int arrow_width= Config.arrowwidth;

		int genlen= 14;
		int genwidth= 10;

		int agglen= 10;
		int aggwidth= 5;

		int xn= offsetX + x;
		int yn= offsetY + y;

		switch (type)
		{
			// association, simple arrow like
			case Link.TYPE_ASSOCIATION :
				switch (info.side)
				{
					// right
					case LinkInfo.SIDE_EAST :
						line(offgraphics, x, y, x + arrow_len, y - arrow_width);
						line(offgraphics, x, y, x + arrow_len, y + arrow_width);
						break;
						// down
					case LinkInfo.SIDE_SOUTH :
						line(offgraphics, x, y, x + arrow_width, y + arrow_len);
						line(offgraphics, x, y, x - arrow_width, y + arrow_len);
						break;
						// left
					case LinkInfo.SIDE_WEST :
						line(offgraphics, x, y, x - arrow_len, y - arrow_width);
						line(offgraphics, x, y, x - arrow_len, y + arrow_width);
						break;
						// up
					case LinkInfo.SIDE_NORTH :
						line(offgraphics, x, y, x + arrow_width, y - arrow_len);
						line(offgraphics, x, y, x - arrow_width, y - arrow_len);
						break;
				}
				return;

				// Generalization, triangle like
			case Link.TYPE_GENERALIZATION :

				switch (info.side)
				{
					// right
					case LinkInfo.SIDE_EAST :
						xarr= new int[] { xn, xn + genlen, xn + genlen };
						yarr= new int[] { yn, yn - genwidth, yn + genwidth };
						break;
						// down
					case LinkInfo.SIDE_SOUTH :
						xarr= new int[] { xn, xn + genwidth, xn - genwidth };
						yarr= new int[] { yn, yn + genlen, yn + genlen };
						break;
						// left
					case LinkInfo.SIDE_WEST :
						xarr= new int[] { xn, xn - genlen, xn - genlen };
						yarr= new int[] { yn, yn - genwidth, yn + genwidth };
						break;
						// up
					case LinkInfo.SIDE_NORTH :
					default :
						xarr= new int[] { xn, xn + genwidth, xn - genwidth };
						yarr= new int[] { yn, yn - genlen, yn - genlen };
						break;
				}

				offgraphics.setColor(Color.white);
				offgraphics.fillPolygon(xarr, yarr, 3);
				offgraphics.setColor(Color.black);
				offgraphics.drawPolygon(xarr, yarr, 3);
				return;

				// Aggregation, diamond shape like
			case Link.TYPE_AGGREGATION :
				switch (info.side)
				{

					// right
					case LinkInfo.SIDE_EAST :
						xarr= new int[] { xn, xn + agglen, xn + 2 * agglen, xn + agglen };
						yarr= new int[] { yn, yn - aggwidth, yn, yn + aggwidth };
						break;
						// down
					case LinkInfo.SIDE_SOUTH :
						xarr= new int[] { xn, xn + aggwidth, xn, xn - aggwidth };
						yarr= new int[] { yn, yn + agglen, yn + 2 * agglen, yn + agglen };
						break;
						// left
					case LinkInfo.SIDE_WEST :
						xarr= new int[] { xn, xn - agglen, xn - 2 * agglen, xn - agglen };
						yarr= new int[] { yn, yn - aggwidth, yn, yn + aggwidth };
						break;
						// up
					case LinkInfo.SIDE_NORTH :
					default :
						xarr= new int[] { xn, xn + aggwidth, xn, xn - aggwidth };
						yarr= new int[] { yn, yn - agglen, yn - 2 * agglen, yn - agglen };
						break;
				}
				offgraphics.setColor(Color.white);
				offgraphics.fillPolygon(xarr, yarr, 4);
				offgraphics.setColor(Color.black);
				offgraphics.drawPolygon(xarr, yarr, 4);
				return;

				// this is a circle arrow end for a note
			case Link.TYPE_COMMENT :
				offgraphics.setColor(Color.white);
				offgraphics.fillOval(xn - 6, yn - 6, 13, 13);

				offgraphics.setColor(Color.black);
				offgraphics.drawOval(xn - 6, yn - 6, 13, 13);
				return;
		}

	}

	/** This enables and disables the scrolling. */
	public void enableScrollEvents(boolean enable)
	{
		allowScrollEvents= enable;
	}

	/**
	 * Exports the whole diagram as text file for later loading in designlets.
	 */
	public void export(String fname)
	{
		System.out.println("exporting to " + fname);
		try
		{
			FileWriter fw= new FileWriter(fname);
			fw.write(shapes.size() + "\n");
			for (Iterator iter= shapes.iterator(); iter.hasNext();)
			{
				Shape s= (Shape)iter.next();
				s.export(fw);
			}

			fw.write(links.size() + "\n");

			for (Iterator iter= links.iterator(); iter.hasNext();)
			{
				Link l= (Link)iter.next();
				fw.write(l.from.shape.getName() + "\n");
				fw.write(l.to.shape.getName() + "\n");
				fw.write(l.type + "\n");
			}

			fw.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	/** Looks for the shape of a java class. */
	public Shape getShape(JavaClass javaclass)
	{
		return javaclass.getShape();
	}

	/**
	 * Looks for the shape of a java class, name is fully
	 * qualified with package identifiers.
	 */

	public Shape getShape(Point p)
	{
		Shape result= null;
		for (Iterator iter= shapes.iterator(); iter.hasNext();)
		{
			Shape n= (Shape)iter.next();
			if (n.contains(p))
			{
				result= n;
			}
		}
		return result;
	}

	/** Find a linked neighbour shape. */
	public Shape getShape(Shape n)
	{
		for (Iterator iter= links.iterator(); iter.hasNext();)
		{
			Link l= (Link)iter.next();
			Shape n0= (Shape)l.from.shape;
			if (n == n0)
				return l.to.shape;
			n0= (Shape)l.to.shape;
			if (n == n0)
				return l.from.shape;
		}
		return null;
	}

	/** Looks for a the shape of a java class, name is fully
	 *  qualified with package identifiers.
	 */

	public Shape getShape(String name)
	{

		Shape result= null;
		for (Iterator iter= shapes.iterator(); iter.hasNext();)
		{
			Shape n= (Shape)iter.next();
			if (n.getName().equals(name))
			{
				result= n;
			}
		}
		return result;
	}

	/**
	 * Imports the whole diagram from a text file (loading in a designlets).
	 */
	public void imports(BufferedReader br)
	{
		try
		{
			int len= Integer.parseInt(br.readLine());
			for (int n= 0; n < len; n++)
			{
				Shape s= new Shape();
				s.init(this, "", 0);
				s.imports(br);
				shapes.add(s);
			}

			len= Integer.parseInt(br.readLine());
			//System.out.println("Links:" + len);
			for (int n= 0; n < len; n++)
			{
				String from= br.readLine();
				String to= br.readLine();
				int type= Integer.parseInt(br.readLine());
				//System.out.println("adding link:<" + from +">  <" + to + ">  <" + type + ">");
				setLinkHint(from, to, type);
			}

			br.close();

			repaint();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	/** Initializes the diagrampanel, fontmetrics and eventlisteners. */
	public int init()
	{
		//setDimension(getSize());

		setLayout(new BorderLayout());

		add("Center", innerpanel);
		add("South", sbarhorizontal);
		add("East", sbarvertical);

		// scrollbars
		sbarhorizontal.setValues(0, 10, 0, 110);
		sbarvertical.setValues(0, 10, 0, 110);
		sbarhorizontal.addAdjustmentListener(this);
		sbarvertical.addAdjustmentListener(this);

		createpopups();

		// event handling
		addMouseListener(new MyMouseAdapter());
		addKeyListener(new MyKeyAdapter());

		myFont= new java.awt.Font(Config.fontname, Font.PLAIN, Config.fontsize);
		if (myFont == null)
		{
			ErrorSystem.error("could not load font:" + Config.fontname);
			ErrorSystem.error("try default font Times,9.");
			myFont= new java.awt.Font("Times", Font.PLAIN, 9);
			if (myFont == null)
			{
				return ErrorSystem.FAILED;
			}
		}
		else
			setFont(myFont);

		fm= getFontMetrics(myFont);

		if (fm == null)
		{
			ErrorSystem.error("could not initialize java font metrics of font " + Config.fontname);
			return ErrorSystem.FAILED;
		}

		else
			ErrorSystem.info("font metrics initialized");

		requestFocus();

		return ErrorSystem.OK;
	}

	/** Need it to get the keyboard focus. */
	public boolean isFocusTraversable()
	{
		return true;
	}

	/** Line drawing with respect to the offset position of the view */
	private void line(Graphics offgraphics, int x1, int y1, int x2, int y2)
	{
		offgraphics.drawLine(offsetX + x1, offsetY + y1, offsetX + x2, offsetY + y2);
	}

	/** Paints arrows and stuff */
	private void paintLink(Graphics offgraphics, LinkInfo from, LinkInfo to)
	{
		// from.node ist gegenberliegende Node
		int dx= to.connector.x - from.connector.x;
		int dy= to.connector.y - from.connector.y;

		boolean gegenueber= false;

		int px= (int)dx;
		int py= (int)dy;

		// gegenueberliegende Seiten ?
		if ((Math.abs(to.side - from.side) % 2) == 0)
			gegenueber= true;

		if (gegenueber)
		{
			px= (int)dx / 2;
			py= (int)dy / 2;

			// waagerecht, d.h. senkrechte Verbindung auf x+dx/2
			if ((to.side % 2) == 0)
				offgraphics.drawLine(
					offsetX + (int)from.connector.x + px,
					offsetY + (int)from.connector.y,
					offsetX + (int)from.connector.x + px,
					offsetY + (int)to.connector.y);
			// senkrecht, senkrechte Verbindung
			else
				offgraphics.drawLine(
					offsetX + (int)from.connector.x,
					offsetY + (int)from.connector.y + py,
					offsetX + (int)to.connector.x,
					offsetY + (int)from.connector.y + py);
		}

		switch (from.side)
		{
			// links
			case 0 :
				offgraphics.drawLine(
					offsetX + (int)from.connector.x,
					offsetY + (int)from.connector.y,
					offsetX + (int)from.connector.x + px,
					offsetY + (int)from.connector.y);
				break;
				// rechts
			case 2 :
				offgraphics.drawLine(
					offsetX + (int)from.connector.x,
					offsetY + (int)from.connector.y,
					offsetX + (int)from.connector.x + px,
					offsetY + (int)from.connector.y);
				break;
				// unten
			case 1 :
				offgraphics.drawLine(
					offsetX + (int)from.connector.x,
					offsetY + (int)from.connector.y,
					offsetX + (int)from.connector.x,
					offsetY + (int)from.connector.y + py);
				break;
				// oben
			case 3 :
				offgraphics.drawLine(
					offsetX + (int)from.connector.x,
					offsetY + (int)from.connector.y,
					offsetX + (int)from.connector.x,
					offsetY + (int)from.connector.y + py);
				break;
		}

		switch (to.side)
		{
			case 0 :
				line(offgraphics, to.connector.x, to.connector.y, to.connector.x - px, to.connector.y);

				break;
			case 2 :
				line(offgraphics, to.connector.x, to.connector.y, to.connector.x - px, to.connector.y);

				break;

			case 1 :
				line(offgraphics, to.connector.x, to.connector.y, to.connector.x, to.connector.y - py);

				break;

			case 3 :
				line(offgraphics, to.connector.x, to.connector.y, to.connector.x, to.connector.y - py);

				break;
		}
	}

	/** Paints the end of the links */
	private void paintLinkInfo(LinkInfo info, Graphics g)
	{
		int o= 3;
		int r= 6;

		g.fillOval(offsetX + info.connector.x - o, offsetY + info.connector.y - o, r, r);
	}

	/** Removes the link between 2 selected java classes. */
	public void removeLink()
	{
		removeLink((Shape)selection.get(0), (Shape)selection.get(1));
	}

	/** Removes the link between 2 java classes. */
	public void removeLink(Shape n0, Shape n1)
	{

		for (Iterator iter= links.iterator(); iter.hasNext();)
		{
			Link l= (Link)iter.next();
			if ((l.from.shape == n0) && (l.to.shape == n1))
			{
				links.remove(l);
				break;
			}
		}
		repaint();
	}

	/** This adds the comment-shapes that are saved in the project. */
	public void setCommentHint(String comment, String name, String to, int x, int y, int lines)
	{

		if (getShape(to) != null)
		{
			updateCommentShape(ShapeInfo.createFromString(comment, lines), getShape(to), true).setMidLocation(x, y);
		}
	}

	/** Inits the usual offscreen graphics and images etc) */
	private void setDimension(Dimension d)
	{

		offscreensize= d;
		offscreen= createImage(d.width, d.height);

		offgraphics= offscreen.getGraphics();
		offgraphics.setFont(myFont);
		vrect.setDirty();
	}

	/**
	 * Creates a link between two shapes given by their java names.
	 * Valid types are "association" and "generalization".
	 */

	public void setLinkHint(String from, String to, int type1)
	{
		Shape n1= getShape(from);
		Shape n2= getShape(to);
		addLink(n1, n2, type1);
		repaint();
	}

	/** Switches betweeen scrolling and moving of shapes */
	private void setMouseMode(int mode)
	{
		mousemode= mode;
	}

	/** Scrolling procedure. */
	public void setScrollerValues(int x, int y)
	{
		if (x < 0)
			x= 0;
		else
			if (x > 100)
				x= 100;

		if (y < 0)
			y= 0;
		else
			if (y > 100)
				y= 100;

		sbarhorizontal.setValue(x);
		sbarvertical.setValue(y);
	}

	/** Gives a shape its position when it was found during loading. */
	public void setShapeHint(JavaProject.ShapeHint sh)
	{
		for (Iterator iter= shapes.iterator(); iter.hasNext();)
		{
			Shape shape= (Shape)iter.next();
			if (shape == null)
				return;
			if (shape.getName().equals(sh.name))
			{
				shape.setMidLocation(sh.x, sh.y);
				shape.isexpanded= sh.isexpanded;
				shape.setShapeColor(sh.color);
				shape.fixsize();
			}
		}
	}

	/**
	 * Sets the view offsets, (x and y between 0 and 99).
	 * This surely could be done a better way .
	 *
	 */

	public void setViewArea(int x, int y)
	{

		Dimension d1= vrect.getSize();
		int x1= d1.width - drawareasize.width;
		int y1= d1.height - drawareasize.height;

		// gets upper left point
		Point pos= vrect.getLocation();

		if (x1 > 0)
			offsetX= - (pos.x + (x * x1) / 100);
		if (y1 > 0)
			offsetY= - (pos.y + (y * y1) / 100);
		repaint();
	}

	/** call this when you changed the content of the comment shape */
	public Shape updateCommentShape(ShapeInfo ti[], Shape ashape, boolean isnew)
	{
		Shape shape;
		if (isnew)
		{
			shape= new Shape();
			shapes.add(shape);
			addLink(shape, ashape, 3);
			shape.setMidLocation(ashape.getMidLocation());
		}
		else
		{
			shape= ashape;
		}

		shape.init(this, "comment_" + shapeid++, 1);
		shape.setShapeInfo(ti);

		vrect.setDirty();
		repaint();
		return shape;
	}

	/** Writes all shape and link hints to the coffeetronix project file.
	 *  It's a simple XML-format.
	 */
	public void writeHints(FileWriter filewriter) throws java.io.IOException
	{

		// classhints or interfaces
		for (Iterator iter= shapes.iterator(); iter.hasNext();)
		{
			Shape node= (Shape)iter.next();
			if (node.getType() != 1)
			{

				filewriter.write("\t<shapehint name=\"");
				filewriter.write(node.getName());
				filewriter.write(
					"\" x=\""
						+ node.getMidLocation().x
						+ "\" y=\""
						+ node.getMidLocation().y
						+ "\" expanded=\""
						+ (node.isexpanded ? "true" : "false")
						+ "\" color=\""
						+ node.getShapeColor()
						+ "\" />\n");
			}
		}

		// linkhints
		for (Iterator iter= links.iterator(); iter.hasNext();)
		{
			Link l= (Link)iter.next();

			// dont save automatic created comment links
			if (l.type != Link.TYPE_COMMENT)
			{
				filewriter.write("\t<linkhint from=\"");
				filewriter.write(l.from.shape.getName());
				filewriter.write("\" to=\"");
				filewriter.write(l.to.shape.getName());
				filewriter.write("\" type=\"");

				switch (l.type)
				{
					case 0 :
						filewriter.write("association");
						break;
					case 1 :
						filewriter.write("generalization");
						break;
					case 2 :
						filewriter.write("nesting");
						break;
				}
				filewriter.write("\" />\n");
			}
		}

		// save all comment shapes
		for (Iterator iter= shapes.iterator(); iter.hasNext();)
		{
			Shape shape= (Shape)iter.next();

			if (shape.getType() == Shape.TYPE_COMMENT)
			{
				filewriter.write("\t<comment name=\"");
				filewriter.write(shape.getName());
				filewriter.write("\" to=\"" + getShape(shape).getName());
				filewriter.write(
					"\" x=\"" + shape.getMidLocation().x + "\" y=\"" + shape.getMidLocation().y + "\" show=\"true\">");

				ShapeInfo ti[]= shape.getData();
				for (int n= 0; n < ti.length; n++)
				{
					filewriter.write(ti[n].getText() + "<br/>");
				}
				filewriter.write("</comment>\n");
			}
		}

	}

	public void exportgfx()
	{
		try
		{
			System.out.println("Exporting graphics...");
			System.out.println("Size:" + vrect.getSize().getWidth() + " " + vrect.getSize().getHeight());
			BufferedImage img= ImageIO.read(new File("c://1.jpg"));

			int breite= (int)vrect.getSize().getWidth();
			int hoehe= (int)vrect.getSize().getHeight();

			BufferedImage bi= new BufferedImage(breite, hoehe, BufferedImage.TYPE_INT_ARGB);
			Graphics g= bi.getGraphics();

			drawareasize.setSize(breite,hoehe);
			
			innerpanel.exportdraw(g);

			ImageIO.write(bi, "png", new File("c://test.png"));

		}
		catch (IOException e1)
		{
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

	}
}
