package coffeetronix.diagram;

import java.io.BufferedReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringReader;

import coffeetronix.ErrorSystem;


/** 
 * 
 * Holds Shape Info's for an entry in a classdiagramm-shape ready for drawing.
 * Thes Infos are the type-icon (classname, method, attribute, separator 
 * and things like ), the protection icon and the text to draw.
 * 
 * @author Rene Stoeckel (rene.stoeckel-at-web.de)
 * 
 */

public class ShapeInfo
{	
	/** Shape info type definition */
	public final static int TYPE_NORMAL_TEXT = 0;
	/** Shape info type definition */
	public final static int TYPE_METHOD = 1;
	/** Shape info type definition */
	public final static int TYPE_ATTRIBUTE = 2;
	/** Shape info type definition */
	public final static int TYPE_CLASS_NAME = 3;
	/** Shape info type definition */
	public final static int TYPE_INTERFACE_NAME = 4;
	/** Shape info type definition */
	public final static int TYPE_SEPARATOR = 10;
	
	
	/** Shape info protection definition */
	public final static int PROT_PUBLIC = 0;
	/** Shape info protection definition */
	public final static int PROT_PROTECTED = 1;
	/** Shape info protection definition */
	public final static int PROT_PRIVATE = 2;
	/** Shape info protection definition */
	public final static int PROT_UNKNOWN = 4;

	/** The shape info type */
	private int type;
	/** The shape info field protection type.*/	
	private int protection;
	/** The shape info text. */	
	private String text = "";


	/**
	 * Exports the ShapeInfo text to a textfile for use in designlets.
	 */ 
	
	public void export(FileWriter fw) throws IOException
	{
		fw.write(type + "\n");
		fw.write(protection + "\n");
		fw.write(text + "\n");
	}

	/** 
	 * Imports the shape textinfo from a textfile. 
	 */
	public void imports(BufferedReader br) throws IOException
	{
		type = Integer.parseInt(br.readLine());
		protection  = Integer.parseInt(br.readLine());
		text = br.readLine();
	}

	/** Init Constructor. */
	public ShapeInfo(String text, int protection, int type)
	{
		this.text = text;
		this.protection = protection;
		this.type = type;
	}

	public String getText()
	{
		return text;
	}

	public int getProtection()
	{
		return protection;
	}

	public int getType()
	{
		return type;
	}


	/** Reads multiple lines from a string 
	 * (e.g. additional separators) 
	 * and converts the shape info fields. Used for the comment shape only. 
	 */
	public static ShapeInfo[]
		createFromString(String s, int lines)
	{
		ShapeInfo ti[] = new ShapeInfo[lines];

		BufferedReader reader = new BufferedReader(
			new StringReader(s));

		for (int index = 0; index < lines; index++)
		{
			try
			{
				// beware. last line sometimes returns null (JTextArea bug?)							
				ti[index] = new ShapeInfo("" + reader.readLine(), 0, 0);
			}
			catch(Exception e)
			{
				ErrorSystem.error("TextInfo.createFromString():" + e);
			}
		}
		return ti;
	}
}
